package caisar

  1. Overview
  2. Docs

Module Onnx.TypeProto

Types

The standard ONNX data types.

Sourcemodule Tensor : sig ... end
Sourcemodule Sequence : sig ... end

repeated T

Sourcemodule Map : sig ... end

map<K,V>

Sourcemodule Optional : sig ... end

wrapper for Tensor, Sequence, or Map

Sourcemodule SparseTensor : sig ... end
Sourcetype t = {
  1. value : [ `not_set | `Tensor_type of Tensor.t | `Sequence_type of Sequence.t | `Map_type of Map.t | `Sparse_tensor_type of SparseTensor.t | `Optional_type of Optional.t ];
    (*

    `Tensor_type

    The type of a tensor.

    `Sequence_type

    The type of a sequence.

    NOTE: DNN-only implementations of ONNX MAY elect to not support non-tensor values

        as input and output to graphs and nodes. These types are needed to naturally
        support classical ML operators.  DNN operators SHOULD restrict their input
        and output types to tensors.
    

    `Map_type

    The type of a map.

    `Sparse_tensor_type

    Type of the sparse tensor

    `Optional_type

    The type of an optional.

    *)
  2. denotation : string option;
    (*

    An optional denotation can be used to denote the whole type with a standard semantic description as to what is stored inside. Refer to https://github.com/onnx/onnx/blob/master/docs/TypeDenotation.md#type-denotation-definition for pre-defined type denotations.

    *)
}
Sourceval make : ?value: [ `not_set | `Tensor_type of Tensor.t | `Sequence_type of Sequence.t | `Map_type of Map.t | `Sparse_tensor_type of SparseTensor.t | `Optional_type of Optional.t ] -> ?denotation:string -> unit -> t

Helper function to generate a message using default values

Serialize the message to binary format

Deserialize from binary format

Serialize to Json (compatible with Yojson.Basic.t)

Deserialize from Json (compatible with Yojson.Basic.t)

Sourceval name : unit -> string

Fully qualified protobuf name of this message