package caisar

  1. Overview
  2. Docs

Module TypeProto.OptionalSource

wrapper for Tensor, Sequence, or Map

Sourcetype t = TypeProto.t option

The type and optional shape of the element wrapped. This field MUST be present for this version of the IR. Possible values correspond to OptionalProto.DataType enum

Sourceval make : ?elem_type:TypeProto.t -> unit -> t

Helper function to generate a message using default values

Serialize the message to binary format

Deserialize from binary format

Serialize to Json (compatible with Yojson.Basic.t)

Deserialize from Json (compatible with Yojson.Basic.t)

Sourceval name : unit -> string

Fully qualified protobuf name of this message